@@ -1,5 +1,6 @@
 from __future__ import absolute_import
 from __future__ import print_function
+import ast
 import collections
 import re
 import sys
@@ -20,6 +21,7 @@ def _open(filename):
     return open(filename, 'rb')
 
 
+ENGINE_PREFIX = re.compile(r'^\s*([a-z\-_]+):\s*')
 WHITESPACE = re.compile(u"\s+")
 EXPRESSION = re.compile(u"\s*\${(.*?)}\s*")
 UNDERSCORE_CALL = re.compile("_\(.*\)")
@@ -124,8 +126,9 @@ class Extractor(ElementProgram):
                         self.add_message(msgid, u'Default: %s' % plain_attrs[attr])
 
             for (attribute, value) in attributes.items():
+                value = decode_htmlentities(value)
                 for source in self.get_code_for_attribute(attribute, value):
-                    self.parse_python(decode_htmlentities(source))
+                    self.parse_python(source)
 
         for child in children:
             self.visit(*child)
@@ -151,18 +154,38 @@ class Extractor(ElementProgram):
         self.messages.append(Message(None, msgid, None, [], comment, u'',
             (self.filename, self.linenumber)))
 
+    def _assert_valid_python(self, value):
+        if not is_valid_python(value):
+            print('Aborting due to Python syntax error in %s[%d]: %s',
+                    self.filename, self.linenumber, value)
+            sys.exit(1)
+
     def get_code_for_attribute(self, attribute, value):
         if attribute[0] == TAL_NS:
             if attribute[1] in ['content', 'replace']:
-                yield value
+                (engine, value) = get_tales_engine(value)
+                if engine == 'python':
+                    self._assert_valid_python(value)
+                    yield value
             if attribute[1] == 'define':
                 for (scope, var, value) in parse_defines(value):
-                    yield value
+                    (engine, value) = get_tales_engine(value)
+                    if engine == 'python':
+                        self._assert_valid_python(value)
+                        yield value
             elif attribute[1] == 'repeat':
-                yield value.split(None, 1)[1]
+                (engine, value) = get_tales_engine(value.split(None, 1)[1])
+                if engine == 'python':
+                    self._assert_valid_python(value)
+                    yield value
         else:
-            for source in EXPRESSION.findall(value):
-                yield source
+            try:
+                for source in get_python_expressions(value):
+                    yield source
+            except SyntaxError:
+                print('Aborting due to Python syntax error in %s[%d]: %s',
+                        self.filename, self.linenumber, value)
+                sys.exit(1)
 
     def parse_python(self, source):
         if not isinstance(source, bytes):
@@ -172,6 +195,48 @@ class Extractor(ElementProgram):
                 location=(self.filename, self.linenumber + message.location[1])))
 
 
+def is_valid_python(source):
+    try:
+        ast.parse(source, mode='eval')
+    except SyntaxError:
+        return False
+    else:
+        return True
+
+
+def get_tales_engine(source):
+    m = ENGINE_PREFIX.match(source)
+    if m is None:
+        return ('python', source)
+    else:
+        return (m.group(1), source[m.end():])
+
+
+def get_python_expressions(source):
+    regex = re.compile(r'(?<!\\)\$({(?P<expression>.*)})', re.DOTALL)
+    while source:
+        m = regex.search(source)
+        if m is None:
+            break
+
+        source = source[m.start():]
+        matched = m.group(0)
+
+        m = regex.search(source)
+        while m is not None:
+            candidate = m.group('expression')
+            if is_valid_python(candidate):
+                yield candidate
+                source = source[m.end():]
+                break
+            else:
+                matched = matched[:-1]
+                m = regex.search(matched)
+        if m is None:
+            # We found ${, but could not find a valid python expression
+            raise SyntaxError()
+
+
 @register_extractor('xml', ['.pt', '.zpt'])
 def extract_xml(filename, options):
     extractor = Extractor(filename, options)
